@@ -3,6 +3,7 @@
 import pprint
 import re
 import os, sys
+import io
 import unittest
 
 sys.path[0:0] = ['.', '..']
@@ -118,18 +119,18 @@ class TestCParser_fundamentals(TestCParser_base):
 
     def test_FileAST(self):
         t = self.parse('int a; char c;')
-        self.assertTrue(isinstance(t, FileAST))
+        self.assertIsInstance(t, FileAST)
         self.assertEqual(len(t.ext), 2)
 
         # empty file
         t2 = self.parse('')
-        self.assertTrue(isinstance(t2, FileAST))
+        self.assertIsInstance(t2, FileAST)
         self.assertEqual(len(t2.ext), 0)
 
     def test_empty_toplevel_decl(self):
         code = 'int foo;;'
         t = self.parse(code)
-        self.assertTrue(isinstance(t, FileAST))
+        self.assertIsInstance(t, FileAST)
         self.assertEqual(len(t.ext), 1)
         self.assertEqual(self.get_decl(code),
             ['Decl', 'foo',
@@ -393,6 +394,14 @@ class TestCParser_fundamentals(TestCParser_base):
                             ['TypeDecl', ['IdentifierType', ['int']]]]],
                         ['TypeDecl', ['IdentifierType', ['int']]]]]])
 
+        self.assertEqual(self.get_decl('int (*k)(const volatile int* q);'),
+            ['Decl', 'k',
+                ['PtrDecl',
+                    ['FuncDecl',
+                        [['Decl', ['const', 'volatile'], 'q',
+                            ['PtrDecl', ['TypeDecl', ['IdentifierType', ['int']]]]]],
+                        ['TypeDecl', ['IdentifierType', ['int']]]]]])
+
         # restrict qualifier
         self.assertEqual(self.get_decl('int (*k)(restrict int* q);'),
             ['Decl', 'k',
@@ -450,7 +459,7 @@ class TestCParser_fundamentals(TestCParser_base):
         d2 = "static char * const p;"
         assert_qs(d2, 0, [], ['static'])
         pdecl = self.parse(d2).ext[0].type
-        self.assertTrue(isinstance(pdecl, PtrDecl))
+        self.assertIsInstance(pdecl, PtrDecl)
         self.assertEqual(pdecl.quals, ['const'])
 
     def test_sizeof(self):
@@ -468,9 +477,9 @@ class TestCParser_fundamentals(TestCParser_base):
         compound = self.parse(e).ext[0].body
 
         s1 = compound.block_items[0].init
-        self.assertTrue(isinstance(s1, UnaryOp))
+        self.assertIsInstance(s1, UnaryOp)
         self.assertEqual(s1.op, 'sizeof')
-        self.assertTrue(isinstance(s1.expr, ID))
+        self.assertIsInstance(s1.expr, ID)
         self.assertEqual(s1.expr.name, 'k')
 
         s2 = compound.block_items[1].init
@@ -496,15 +505,15 @@ class TestCParser_fundamentals(TestCParser_base):
             """
         compound = self.parse(e).ext[0].body
         s1 = compound.block_items[0].init
-        self.assertTrue(isinstance(s1, FuncCall))
-        self.assertTrue(isinstance(s1.name, ID))
+        self.assertIsInstance(s1, FuncCall)
+        self.assertIsInstance(s1.name, ID)
         self.assertEqual(s1.name.name, 'offsetof')
-        self.assertTrue(isinstance(s1.args.exprs[0], Typename))
-        self.assertTrue(isinstance(s1.args.exprs[1], ID))
+        self.assertIsInstance(s1.args.exprs[0], Typename)
+        self.assertIsInstance(s1.args.exprs[1], ID)
         s3 = compound.block_items[2].init
-        self.assertTrue(isinstance(s3.args.exprs[1], StructRef))
+        self.assertIsInstance(s3.args.exprs[1], StructRef)
         s4 = compound.block_items[3].init
-        self.assertTrue(isinstance(s4.args.exprs[1], ArrayRef))
+        self.assertIsInstance(s4.args.exprs[1], ArrayRef)
 
     def test_compound_statement(self):
         e = """
@@ -512,7 +521,7 @@ class TestCParser_fundamentals(TestCParser_base):
             }
             """
         compound = self.parse(e).ext[0].body
-        self.assertTrue(isinstance(compound, Compound))
+        self.assertIsInstance(compound, Compound)
         self.assert_coord(compound, 2)
 
     # The C99 compound literal feature
@@ -542,21 +551,21 @@ class TestCParser_fundamentals(TestCParser_base):
         e1 = "enum mycolor op;"
         e1_type = self.parse(e1).ext[0].type.type
 
-        self.assertTrue(isinstance(e1_type, Enum))
+        self.assertIsInstance(e1_type, Enum)
         self.assertEqual(e1_type.name, 'mycolor')
         self.assertEqual(e1_type.values, None)
 
         e2 = "enum mysize {large=20, small, medium} shoes;"
         e2_type = self.parse(e2).ext[0].type.type
 
-        self.assertTrue(isinstance(e2_type, Enum))
+        self.assertIsInstance(e2_type, Enum)
         self.assertEqual(e2_type.name, 'mysize')
 
         e2_elist = e2_type.values
-        self.assertTrue(isinstance(e2_elist, EnumeratorList))
+        self.assertIsInstance(e2_elist, EnumeratorList)
 
         for e2_eval in e2_elist.enumerators:
-            self.assertTrue(isinstance(e2_eval, Enumerator))
+            self.assertIsInstance(e2_eval, Enumerator)
 
         self.assertEqual(e2_elist.enumerators[0].name, 'large')
         self.assertEqual(e2_elist.enumerators[0].value.value, '20')
@@ -574,12 +583,12 @@ class TestCParser_fundamentals(TestCParser_base):
             """
 
         e3_type = self.parse(e3).ext[0].type.type
-        self.assertTrue(isinstance(e3_type, Enum))
+        self.assertIsInstance(e3_type, Enum)
         e3_elist = e3_type.values
-        self.assertTrue(isinstance(e3_elist, EnumeratorList))
+        self.assertIsInstance(e3_elist, EnumeratorList)
 
         for e3_eval in e3_elist.enumerators:
-            self.assertTrue(isinstance(e3_eval, Enumerator))
+            self.assertIsInstance(e3_eval, Enumerator)
 
         self.assertEqual(e3_elist.enumerators[0].name, 'red')
         self.assertEqual(e3_elist.enumerators[0].value, None)
@@ -998,6 +1007,34 @@ class TestCParser_fundamentals(TestCParser_base):
         self.assertEqual(parsed_struct.type.type.decls[0].bitsize.value, '6')
         self.assertEqual(parsed_struct.type.type.decls[1].bitsize.value, '2')
 
+    def test_struct_empty(self):
+        """
+        Tests that parsing an empty struct works.
+
+        Empty structs do NOT follow C99 (See 6.2.5-20 of the C99 standard).
+        This is nevertheless supported by some compilers (clang, gcc),
+        especially when using FORTIFY code.
+        Some compilers (visual) will fail to compile with an error.
+        """
+        # an empty struct. This is NOT C99 compliant
+        s1 = """
+                 struct foo { };
+             """
+
+        parsed_struct = self.parse(s1).ext[0]
+        self.assertEqual(expand_decl(parsed_struct),
+                         ['Decl', None, ['Struct', 'foo', []]])
+
+        s2 = """struct { } foo;"""
+        parsed_struct = self.parse(s2).ext[0]
+        self.assertEqual(expand_decl(parsed_struct),
+                         ['Decl', 'foo', ['TypeDecl', ['Struct', None, []]]])
+
+        s3 = """union { } foo;"""
+        parsed_struct = self.parse(s3).ext[0]
+        self.assertEqual(expand_decl(parsed_struct),
+                         ['Decl', 'foo', ['TypeDecl', ['Union', None, []]]])
+
     def test_tags_namespace(self):
         """ Tests that the tags of structs/unions/enums reside in a separate namespace and
             can be named after existing types.
@@ -1051,7 +1088,7 @@ class TestCParser_fundamentals(TestCParser_base):
 
         s3_type = self.parse(s3).ext[1].type
 
-        self.assertTrue(isinstance(s3_type, Enum))
+        self.assertIsInstance(s3_type, Enum)
         self.assertEqual(s3_type.name, 'mytag')
 
     def test_multi_decls(self):
@@ -1179,6 +1216,30 @@ class TestCParser_fundamentals(TestCParser_base):
                  ['Constant', 'int', '5']],
                 ['Constant', 'int', '6']])
 
+        d5 = 'float d = 1.0;'
+        self.assertEqual(self.get_decl_init(d5),
+            ['Constant', 'double', '1.0'])
+
+        d51 = 'float ld = 1.0l;'
+        self.assertEqual(self.get_decl_init(d51),
+            ['Constant', 'long double', '1.0l'])
+
+        d52 = 'float ld = 1.0L;'
+        self.assertEqual(self.get_decl_init(d52),
+            ['Constant', 'long double', '1.0L'])
+
+        d53 = 'float ld = 1.0f;'
+        self.assertEqual(self.get_decl_init(d53),
+            ['Constant', 'float', '1.0f'])
+
+        d54 = 'float ld = 1.0F;'
+        self.assertEqual(self.get_decl_init(d54),
+            ['Constant', 'float', '1.0F'])
+
+        d55 = 'float ld = 0xDE.38p0;'
+        self.assertEqual(self.get_decl_init(d55),
+            ['Constant', 'float', '0xDE.38p0'])
+
     def test_decl_named_inits(self):
         d1 = 'int a = {.k = 16};'
         self.assertEqual(self.get_decl_init(d1),
@@ -1337,8 +1398,8 @@ class TestCParser_fundamentals(TestCParser_base):
                 int var2[*];
             }
         ''')
-        self.assertTrue(isinstance(ps2.ext[0].body.block_items[1].type.dim, Assignment))
-        self.assertTrue(isinstance(ps2.ext[0].body.block_items[2].type.dim, ID))
+        self.assertIsInstance(ps2.ext[0].body.block_items[1].type.dim, Assignment)
+        self.assertIsInstance(ps2.ext[0].body.block_items[2].type.dim, ID)
 
     def test_pragma(self):
         s1 = r'''
@@ -1348,20 +1409,75 @@ class TestCParser_fundamentals(TestCParser_base):
                 for(;;) {}
                 #pragma
             }
+            struct s {
+            #pragma baz
+            } s;
             '''
         s1_ast = self.parse(s1)
-        self.assertTrue(isinstance(s1_ast.ext[0], Pragma))
+        self.assertIsInstance(s1_ast.ext[0], Pragma)
         self.assertEqual(s1_ast.ext[0].string, 'bar')
         self.assertEqual(s1_ast.ext[0].coord.line, 2)
 
-        self.assertTrue(isinstance(s1_ast.ext[1].body.block_items[0], Pragma))
+        self.assertIsInstance(s1_ast.ext[1].body.block_items[0], Pragma)
         self.assertEqual(s1_ast.ext[1].body.block_items[0].string, 'foo')
         self.assertEqual(s1_ast.ext[1].body.block_items[0].coord.line, 4)
 
-        self.assertTrue(isinstance(s1_ast.ext[1].body.block_items[2], Pragma))
+        self.assertIsInstance(s1_ast.ext[1].body.block_items[2], Pragma)
         self.assertEqual(s1_ast.ext[1].body.block_items[2].string, '')
         self.assertEqual(s1_ast.ext[1].body.block_items[2].coord.line, 6)
 
+        self.assertIsInstance(s1_ast.ext[2].type.type.decls[0], Pragma)
+        self.assertEqual(s1_ast.ext[2].type.type.decls[0].string, 'baz')
+        self.assertEqual(s1_ast.ext[2].type.type.decls[0].coord.line, 9)
+
+    def test_pragmacomp_or_statement(self):
+        s1 = r'''
+            void main() {
+                int sum = 0;
+                for (int i; i < 3; i++)
+                    #pragma omp critical
+                    sum += 1;
+
+                while(sum < 10)
+                    #pragma omp critical
+                    sum += 1;
+
+                mylabel:
+                    #pragma foo
+                    sum += 10;
+
+                if (sum > 10)
+                    #pragma bar
+                    sum = 10;
+
+                switch (sum)
+                case 10:
+                    #pragma foo
+                    sum = 20;
+            }
+        '''
+        s1_ast = self.parse(s1)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[1], For)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[1].stmt, Compound)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[1].stmt.block_items[0], Pragma)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[1].stmt.block_items[1], Assignment)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[2], While)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[2].stmt, Compound)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[2].stmt.block_items[0], Pragma)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[2].stmt.block_items[1], Assignment)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[3], Label)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[3].stmt, Compound)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[3].stmt.block_items[0], Pragma)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[3].stmt.block_items[1], Assignment)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[4], If)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[4].iftrue, Compound)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[4].iftrue.block_items[0], Pragma)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[4].iftrue.block_items[1], Assignment)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[5], Switch)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[5].stmt.stmts[0], Compound)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[5].stmt.stmts[0].block_items[0], Pragma)
+        self.assertIsInstance(s1_ast.ext[0].body.block_items[5].stmt.stmts[0].block_items[1], Assignment)
+
 
 class TestCParser_whole_code(TestCParser_base):
     """ Testing of parsing whole chunks of code.
@@ -1576,12 +1692,12 @@ class TestCParser_whole_code(TestCParser_base):
 
     def test_switch_statement(self):
         def assert_case_node(node, const_value):
-            self.assertTrue(isinstance(node, Case))
-            self.assertTrue(isinstance(node.expr, Constant))
+            self.assertIsInstance(node, Case)
+            self.assertIsInstance(node.expr, Constant)
             self.assertEqual(node.expr.value, const_value)
 
         def assert_default_node(node):
-            self.assertTrue(isinstance(node, Default))
+            self.assertIsInstance(node, Default)
 
         s1 = r'''
         int foo(void) {
@@ -1689,7 +1805,7 @@ class TestCParser_whole_code(TestCParser_base):
         testdir = os.path.dirname(__file__)
         name = os.path.join(testdir, 'c_files', name)
         assert os.path.exists(name)
-        return open(name, 'rU')
+        return io.open(name)
 
     def test_whole_file(self):
         # See how pycparser handles a whole, real C file.
@@ -1719,16 +1835,16 @@ class TestCParser_whole_code(TestCParser_base):
             code = f.read()
         p = self.parse(code)
 
-        self.assertTrue(isinstance(p.ext[0], Typedef))
+        self.assertIsInstance(p.ext[0], Typedef)
         self.assertEqual(p.ext[0].coord.line, 213)
         self.assertEqual(p.ext[0].coord.file, r"D:\eli\cpp_stuff\libc_include/stddef.h")
 
-        self.assertTrue(isinstance(p.ext[-1], FuncDef))
+        self.assertIsInstance(p.ext[-1], FuncDef)
         self.assertEqual(p.ext[-1].coord.line, 15)
         self.assertEqual(p.ext[-1].coord.file, "example_c_file.c")
 
-        self.assertTrue(isinstance(p.ext[-8], Typedef))
-        self.assertTrue(isinstance(p.ext[-8].type, TypeDecl))
+        self.assertIsInstance(p.ext[-8], Typedef)
+        self.assertIsInstance(p.ext[-8].type, TypeDecl)
         self.assertEqual(p.ext[-8].name, 'cookie_io_functions_t')
 
 
@@ -1758,7 +1874,7 @@ class TestCParser_typenames(TestCParser_base):
                 unsigned TT;
             }
             '''
-        self.assertTrue(isinstance(self.parse(s2), FileAST))
+        self.assertIsInstance(self.parse(s2), FileAST)
 
     def test_ambiguous_parameters(self):
         # From ISO/IEC 9899:TC2, 6.7.5.3.11:
@@ -1990,7 +2106,7 @@ class TestCParser_typenames(TestCParser_base):
               TT x = 10;
             }
             '''
-        self.assertTrue(isinstance(self.parse(s1), FileAST))
+        self.assertIsInstance(self.parse(s1), FileAST)
 
     def test_samescope_reuse_name(self):
         # a typedef name cannot be reused as an object name in the same scope
